@@ -1,4 +1,4 @@
-EM.group <- function(pars, constrain, PrepList, list, Theta, PROBTRACE, DERIV)
+EM.group <- function(pars, constrain, PrepList, list, Theta, DERIV)
 {
     verbose <- list$verbose
     nfact <- list$nfact
@@ -12,7 +12,7 @@ EM.group <- function(pars, constrain, PrepList, list, Theta, PROBTRACE, DERIV)
     sitems <- list$sitems
     theta <- list$theta
     J <- length(itemloc) - 1L
-    nfullpars <- 0
+    nfullpars <- 0L
     estpars <- c()
     prodlist <- PrepList[[1L]]$prodlist
     gfulldata <- gtheta0 <- gstructgrouppars <- vector('list', ngroups)
@@ -56,25 +56,25 @@ EM.group <- function(pars, constrain, PrepList, list, Theta, PROBTRACE, DERIV)
     redun_constr <- rep(FALSE, length(estpars))
     if(length(constrain) > 0L){
         for(i in 1L:length(constrain)){
-            L[constrain[[i]], constrain[[i]]] <- 1L
+            L[constrain[[i]], constrain[[i]]] <- 1L/length(constrain[[i]])
             for(j in 2L:length(constrain[[i]]))
                 redun_constr[constrain[[i]][j]] <- TRUE
         }
     }
     estindex_unique <- index[estpars & !redun_constr]
-    if(any(diag(L)[!estpars] > 0)){
+    if(any(diag(L)[!estpars] > 0L)){
         redindex <- index[!estpars]
         stop('Constraint applied to fixed parameter(s) ',
-             paste(paste0(redindex[diag(L)[!estpars] > 0]), ''), ' but should only be applied to
+             paste(paste0(redindex[diag(L)[!estpars] > 0L]), ''), ' but should only be applied to
                  estimated parameters. Please fix!')
     }
     Prior <- prior <- Priorbetween <- gstructgrouppars <- rlist <- r <- list()
     #make sure constrained pars are equal
-    tmp <- rowSums(L)
-    tmp[tmp == 0] <- 1L
-    tmp <- matrix(1/tmp, length(longpars), length(longpars), byrow = TRUE)
-    tmp2 <- abs(diag(L) - 1L)
-    longpars <- diag((tmp * L) * longpars) + tmp2 * longpars
+    tmp <- L
+    tmp2 <- diag(tmp)
+    tmp2[tmp2 == 0L] <- 1L
+    diag(tmp) <- tmp2
+    longpars <- as.numeric(tmp %*% longpars)
     LL <- 0
     for(g in 1L:ngroups)
         r[[g]] <- PrepList[[g]]$tabdata[, ncol(PrepList[[g]]$tabdata)]
@@ -95,9 +95,12 @@ EM.group <- function(pars, constrain, PrepList, list, Theta, PROBTRACE, DERIV)
     if(length(constrain) > 0L)
        for(i in 1L:length(constrain))
            est[constrain[[i]][-1L]] <- FALSE
+    names(longpars) <- names(est)
     EMhistory <- matrix(NA, NCYCLES+1L, length(longpars))
     EMhistory[1L,] <- longpars    
     gTheta <- vector('list', ngroups)
+    ANY.PRIOR <- rep(FALSE, ngroups)
+    NO.CUSTOM <- !any(sapply(pars[[1L]], class) %in% 'custom')
     for(g in 1L:ngroups){
         gTheta[[g]] <- Theta 
         if(length(prodlist) > 0L)
@@ -107,7 +110,8 @@ EM.group <- function(pars, constrain, PrepList, list, Theta, PROBTRACE, DERIV)
             prior[[g]] <- dnorm(theta, 0, 1)
             prior[[g]] <- prior[[g]]/sum(prior[[g]])  
         }
-    }
+        ANY.PRIOR[g] <- any(sapply(pars[[g]], function(x) x@any.prior))
+    }    
     preMstep.longpars2 <- preMstep.longpars <- longpars
     accel <- 0
     
@@ -137,9 +141,9 @@ EM.group <- function(pars, constrain, PrepList, list, Theta, PROBTRACE, DERIV)
                 rlist[[g]] <- Estep.bfactor(pars=pars[[g]], tabdata=PrepList[[g]]$tabdata,
                                             Theta=gTheta[[g]], prior=prior[[g]], Prior=Prior[[g]],
                                             Priorbetween=Priorbetween[[g]], specific=specific, sitems=sitems,
-                                            itemloc=itemloc)
+                                            itemloc=itemloc, NO.CUSTOM=NO.CUSTOM)
             } else {
-                rlist[[g]] <- Estep.mirt(pars=pars[[g]], tabdata=PrepList[[g]]$tabdata,
+                rlist[[g]] <- Estep.mirt(pars=pars[[g]], tabdata=PrepList[[g]]$tabdata, NO.CUSTOM=NO.CUSTOM,
                                          Theta=gTheta[[g]], prior=Prior[[g]], itemloc=itemloc)
             }
             LL <- LL + sum(r[[g]]*log(rlist[[g]]$expected))
@@ -152,29 +156,30 @@ EM.group <- function(pars, constrain, PrepList, list, Theta, PROBTRACE, DERIV)
         }
         preMstep.longpars2 <- preMstep.longpars
         preMstep.longpars <- longpars
-        if(all(!est)) break        
+        if(all(!est)) break
         longpars <- Mstep(pars=pars, est=est, longpars=longpars, ngroups=ngroups, J=J,
-                      gTheta=gTheta, itemloc=itemloc, Prior=Prior,
-                      PrepList=PrepList, L=L, UBOUND=UBOUND, LBOUND=LBOUND,
-                      constrain=constrain, cycle=cycles, PROBTRACE=PROBTRACE, DERIV=DERIV)                   
+                          gTheta=gTheta, itemloc=itemloc, Prior=Prior, ANY.PRIOR=ANY.PRIOR, 
+                          NO.CUSTOM=NO.CUSTOM, PrepList=PrepList, L=L, UBOUND=UBOUND, LBOUND=LBOUND,
+                          rlist=rlist, constrain=constrain, cycle=cycles, DERIV=DERIV)                   
         if(list$accelerate && cycles > 10L && cycles %% 3 == 0L){            
             dX2 <- preMstep.longpars - preMstep.longpars2
             dX <- longpars - preMstep.longpars
             d2X2 <- dX - dX2
-            accel <- 1 - sqrt((dX2 %*% dX2) / (d2X2 %*% d2X2))
-            if(accel < -4) accel <- -4
+            accel <- 1 - sqrt((dX %*% dX) / (d2X2 %*% d2X2))
+            if(accel < -5) accel <- -5
             longpars <- (1 - accel) * longpars + accel * preMstep.longpars
         }        
         pars <- reloadPars(longpars=longpars, pars=pars, ngroups=ngroups, J=J)
         EMhistory[cycles+1L,] <- longpars
         if(verbose)
-            if(cycles > 1L)
-                cat(sprintf('\rIteration: %d, Log-Lik: %.3f, Max-Change: %.5f',
-                            cycles, LL, max(abs(preMstep.longpars - longpars))))
+            cat(sprintf('\rIteration: %d, Log-Lik: %.3f, Max-Change: %.5f',
+                        cycles, LL, max(abs(preMstep.longpars - longpars))))
         if(cycles > 3L && all(abs(preMstep.longpars - longpars) < TOL))  break
     } #END EM
-    if(cycles == NCYCLES)
+    if(cycles == NCYCLES){
         message('EM iterations terminated after ', cycles, ' iterations.')        
+        converge <- 0L
+    }
     infological <- estpars & !redun_constr
     correction <- numeric(length(estpars[estpars & !redun_constr]))
     names(correction) <- names(estpars[estpars & !redun_constr])
@@ -214,29 +219,29 @@ EM.group <- function(pars, constrain, PrepList, list, Theta, PROBTRACE, DERIV)
 }
 
 # Estep for mirt
-Estep.mirt <- function(pars, tabdata, Theta, prior, itemloc, itemtrace=NULL, deriv = FALSE)
+Estep.mirt <- function(pars, tabdata, Theta, prior, itemloc, NO.CUSTOM=FALSE, 
+                       itemtrace=NULL, deriv = FALSE)
 {
-    nfact <- ncol(Theta)
     nquad <- nrow(Theta)
     J <- length(itemloc) - 1L
     r <- tabdata[ ,ncol(tabdata)]
     X <- tabdata[ ,1L:(ncol(tabdata) - 1L), drop = FALSE]
     if(is.null(itemtrace))
-        itemtrace <- computeItemtrace(pars=pars, Theta=Theta, itemloc=itemloc)
-    retlist <- .Call("Estep", itemtrace, prior, X, nfact, r)
+        itemtrace <- computeItemtrace(pars=pars, Theta=Theta, itemloc=itemloc, NO.CUSTOM=NO.CUSTOM)
+    retlist <- .Call("Estep", itemtrace, prior, X, r)
     if(deriv) retlist$itemtrace <- itemtrace
     return(retlist)
 }
 
 # Estep for bfactor
 Estep.bfactor <- function(pars, tabdata, Theta, prior, Prior, Priorbetween, specific, 
-                          sitems, itemloc, itemtrace=NULL)
+                          NO.CUSTOM=FALSE, sitems, itemloc, itemtrace=NULL)
 {
     J <- length(itemloc) - 1L
     r <- tabdata[ ,ncol(tabdata)]
     X <- tabdata[ ,1L:(ncol(tabdata) - 1L)]
     if(is.null(itemtrace))
-        itemtrace <- computeItemtrace(pars=pars, Theta=Theta, itemloc=itemloc)
+        itemtrace <- computeItemtrace(pars=pars, Theta=Theta, itemloc=itemloc, NO.CUSTOM=NO.CUSTOM)
     retlist <- .Call("Estepbfactor", itemtrace, prior, Priorbetween, X, r, sitems)
     r1 <- matrix(0, nrow(Theta), ncol(X))
     for (i in 1L:J){
@@ -247,19 +252,26 @@ Estep.bfactor <- function(pars, tabdata, Theta, prior, Prior, Priorbetween, spec
     return(list(r1=r1, expected=retlist$expected))
 }
 
-Mstep <- function(pars, est, longpars, ngroups, J, gTheta, itemloc, PrepList, L,
-                  UBOUND, LBOUND, constrain, cycle, PROBTRACE, DERIV, Prior){
+Mstep <- function(pars, est, longpars, ngroups, J, gTheta, itemloc, PrepList, L, ANY.PRIOR,
+                  UBOUND, LBOUND, constrain, cycle, DERIV, Prior, rlist, NO.CUSTOM){
     p <- longpars[est]
     opt <- try(optim(p, fn=Mstep.LL, gr=Mstep.grad, method='L-BFGS-B', 
                      control=list(maxit=ifelse(cycle > 10L, 10L, 5L), fnscale = -1L),  
-                     PROBTRACE=PROBTRACE, DERIV=DERIV,
+                     DERIV=DERIV, rlist=rlist, NO.CUSTOM=NO.CUSTOM,
                      est=est, longpars=longpars, pars=pars, ngroups=ngroups, J=J, gTheta=gTheta,
-                     PrepList=PrepList, L=L, constrain=constrain,
+                     PrepList=PrepList, L=L, constrain=constrain, ANY.PRIOR=ANY.PRIOR,
                      UBOUND=UBOUND, LBOUND=LBOUND, itemloc=itemloc, lower=LBOUND[est], upper=UBOUND[est]),
             silent=TRUE)
+#     #uncomment for testing with nlm
+#     opt <- try(optim(f=Mstep.LL, p,
+#                      DERIV=DERIV, rlist=rlist, NO.CUSTOM=NO.CUSTOM,
+#                      est=est, longpars=longpars, pars=pars, ngroups=ngroups, J=J, gTheta=gTheta,
+#                      PrepList=PrepList, L=L, constrain=constrain, ANY.PRIOR=ANY.PRIOR,
+#                      UBOUND=UBOUND, LBOUND=LBOUND, itemloc=itemloc),
+#                silent=TRUE)
     if(is(opt, 'try-error'))
         stop(opt)
-    longpars[est] <- opt$par
+    longpars[est] <- opt$par    
     if(length(constrain) > 0L)
         for(i in 1L:length(constrain))
             longpars[constrain[[i]][-1L]] <- longpars[constrain[[i]][1L]]
@@ -275,44 +287,34 @@ Mstep <- function(pars, est, longpars, ngroups, J, gTheta, itemloc, PrepList, L,
     return(longpars)
 }
 
-Mstep.LL <- function(p, est, longpars, pars, ngroups, J, gTheta, PrepList, L,
-                     constrain, LBOUND, UBOUND, itemloc, PROBTRACE, DERIV){
+Mstep.LL <- function(p, est, longpars, pars, ngroups, J, gTheta, PrepList, L, NO.CUSTOM,
+                     constrain, LBOUND, UBOUND, itemloc, DERIV, rlist, ANY.PRIOR){
     longpars[est] <- p
     if(length(constrain) > 0L)
        for(i in 1L:length(constrain))
            longpars[constrain[[i]][-1L]] <- longpars[constrain[[i]][1L]]
     pars <- reloadPars(longpars=longpars, pars=pars, ngroups=ngroups, J=J)
-    LLs <- numeric(J)
-    LL <- 0        
-    for(g in 1L:ngroups){                
-        for(i in 1L:J)
-            LLs[i] <- LogLikMstep(pars[[g]][[i]], Theta=gTheta[[g]],
-                                  PROBTRACE=PROBTRACE[[g]][[i]])
-        LL <- LL + sum(LLs)
-    }
-    if(is.nan(LL)){
-        for(g in 1L:ngroups){                
-            for(i in 1L:J){
-                if(is.nan(LogLikMstep(pars[[g]][[i]], Theta=gTheta[[g]],
-                                      PROBTRACE=PROBTRACE[[g]][[i]])))
-                    stop('M-step optimizer did not converge. 
-                         Item ', i, ' in group ', g, ' had the following extreme parameters:\n', 
-                         paste0(paste0('  ', names(pars[[g]][[i]]@est), ' = '), 
-                                round(pars[[g]][[i]]@par), 4))
-            }
-        }        
-    } else return(LL)    
+    LLs <- numeric(ngroups)
+    for(g in 1L:ngroups)
+        LLs[g] <- LogLikMstep(pars[[g]], Theta=gTheta[[g]], rs=rlist[[g]], 
+                              itemloc=itemloc, NO.CUSTOM=NO.CUSTOM, any.prior=ANY.PRIOR[g])        
+#     return(-sum(LLs))
+    return(sum(LLs))
 }
 
-LogLikMstep <- function(x, Theta, PROBTRACE){
-    log_itemtrace <- log(PROBTRACE(x=x, Theta=Theta))
-    LL <- sum(x@rs * log_itemtrace)
-    if(x@any.prior) LL <- LL.Priors(x=x, LL=LL)
+LogLikMstep <- function(x, Theta, itemloc, rs, any.prior, NO.CUSTOM){
+    log_itemtrace <- log(computeItemtrace(pars=x, Theta=Theta, itemloc=itemloc, NO.CUSTOM=NO.CUSTOM))
+    LL <- sum(rs$r1 * log_itemtrace)
+    if(any.prior){
+        for(i in 1L:(length(x)-1L))
+            if(x[[i]]@any.prior)
+                LL <- LL.Priors(x=x[[i]], LL=LL)
+    }
     return(LL)
 }
 
-Mstep.grad <- function(p, est, longpars, pars, ngroups, J, gTheta, PrepList, L,
-                       constrain, LBOUND, UBOUND, itemloc, PROBTRACE, DERIV){
+Mstep.grad <- function(p, est, longpars, pars, ngroups, J, gTheta, PrepList, L,  ANY.PRIOR,
+                       constrain, LBOUND, UBOUND, itemloc, DERIV, rlist, NO.CUSTOM){
     longpars[est] <- p
     if(length(constrain) > 0L)
         for(i in 1L:length(constrain))
@@ -327,9 +329,7 @@ Mstep.grad <- function(p, est, longpars, pars, ngroups, J, gTheta, PrepList, L,
             g[ind1:ind2] <- deriv$grad
             ind1 <- ind2 + 1L
         }
-        i <- i + 1L
-        ind2 <- ind1 + length(pars[[group]][[i]]@par) - 1L
-        g[ind1:ind2] <- 0
+        ind2 <- ind1 + length(pars[[group]][[i + 1L]]@par) - 1L
         ind1 <- ind2 + 1L
     }
     grad <- g %*% L
