@@ -4,7 +4,7 @@ LoadPars <- function(itemtype, itemloc, lambdas, zetas, guess, upper, fulldata,
 {    
     customItemNames <- unique(names(customItems))
     if(is.null(customItemNames)) customItemNames <- 'UsElEsSiNtErNaLNaMe'
-    valid.items <- c('Rasch', '1PL', '2PL', '3PL', '3PLu', '4PL', 'graded',
+    valid.items <- c('Rasch', '2PL', '3PL', '3PLu', '4PL', 'graded',
                     'grsm', 'gpcm', 'rsm', 'nominal', 'PC2PL','PC3PL',
                     '2PLNRM', '3PLNRM', '3PLuNRM', '4PLNRM')
     invalid.items <- is.na(match(itemtype, valid.items))
@@ -12,10 +12,12 @@ LoadPars <- function(itemtype, itemloc, lambdas, zetas, guess, upper, fulldata,
       stop(paste("Unknown itemtype", paste(itemtype[invalid.items], collapse=" ")))
     }
     pars <- vector('list', J)
+    guess <- logit(guess)
+    upper <- logit(upper)
     #startvalues
     startvalues <- vector('list', J)
     for(i in 1L:J){
-        if(any(itemtype[i] == c('Rasch', '1PL')) && K[i] == 2L){
+        if(any(itemtype[i] == c('Rasch')) && K[i] == 2L){
             tmpval <- rep(0, nfact)
             tmpval[lambdas[i,] != 0] <- 1
             val <- c(tmpval, zetas[[i]], guess[i], upper[i])
@@ -27,12 +29,6 @@ LoadPars <- function(itemtype, itemloc, lambdas, zetas, guess, upper, fulldata,
             val <- c(tmpval, 0, zetas[[i]])
             names(val) <- c(paste('a', 1L:nfact, sep=''), paste('d', 0L:(K[i]-1L), sep=''))
         }
-        if(itemtype[i] == '1PL' && K[i] > 2L){
-            tmpval <- rep(0, nfact)
-            tmpval[lambdas[i,] != 0] <- 1
-            val <- c(tmpval, zetas[[i]])
-            names(val) <- c(paste('a', 1L:nfact, sep=''), paste('d', 1L:(K[i]-1L), sep=''))
-        }
         if(any(itemtype[i] == c('2PL', '3PL', '3PLu', '4PL'))){
             val <- c(lambdas[i,], zetas[[i]], guess[i], upper[i])
             names(val) <- c(paste('a', 1L:nfact, sep=''), 'd', 'g','u')
@@ -75,7 +71,7 @@ LoadPars <- function(itemtype, itemloc, lambdas, zetas, guess, upper, fulldata,
                             paste('d', 0L:(K[i]-1L), sep=''))
         }
         if(any(itemtype[i] == c('PC2PL','PC3PL'))){
-            val <- c(lambdas[i,], rep(1, nfact), guess[i], 1)
+            val <- c(lambdas[i,], rep(1, nfact), guess[i], 999)
             names(val) <- c(paste('a', 1L:nfact, sep=''), paste('d', 1L:nfact, sep=''), 'g','u')
         }        
         if(all(itemtype[i] != valid.items)) next
@@ -86,7 +82,7 @@ LoadPars <- function(itemtype, itemloc, lambdas, zetas, guess, upper, fulldata,
     for(i in 1L:J){
         if(itemtype[i] == 'Rasch' && K[i] == 2L)
             freepars[[i]] <- c(rep(FALSE,nfact),TRUE,FALSE,FALSE)
-        if(any(itemtype[i] == c('1PL', '2PL', '3PL', '3PLu', '4PL'))){            
+        if(any(itemtype[i] == c('2PL', '3PL', '3PLu', '4PL'))){            
             if(K[i] != 2L) stop(paste0('Item ', i, ' requires exactly 2 unique categories'))
             estpars <- c(estLambdas[i, ], TRUE, FALSE, FALSE)
             if(any(itemtype[i] == c('3PL', '4PL'))) estpars[length(estpars)-1L] <- TRUE
@@ -102,8 +98,6 @@ LoadPars <- function(itemtype, itemloc, lambdas, zetas, guess, upper, fulldata,
         }
         if(itemtype[i] == 'Rasch' && K[i] > 2L)
             freepars[[i]] <- c(rep(FALSE,nfact), FALSE, rep(TRUE, K[i]-1L))
-        if(itemtype[i] == '1PL' && K[i] > 2L)
-            freepars[[i]] <- c(estLambdas[i, ], rep(TRUE, K[i]-1L))
         if(itemtype[i] == 'grsm')
             freepars[[i]] <- c(estLambdas[i, ], rep(TRUE, K[i]))
         if(itemtype[i] == 'graded')
@@ -167,17 +161,17 @@ LoadPars <- function(itemtype, itemloc, lambdas, zetas, guess, upper, fulldata,
     for(i in 1L:J){
         tmp <- c(itemloc[i]:(itemloc[i+1L] - 1L)) #item location
 
-        if(any(itemtype[i] == c('Rasch', '1PL')) && K[i] == 2L){
+        if(any(itemtype[i] == c('Rasch')) && K[i] == 2L){
             pars[[i]] <- new('dich', par=startvalues[[i]], est=freepars[[i]],
                              nfact=nfact,
-                             dat=fulldata[ ,tmp],
                              ncat=2L,
                              nfixedeffects=nfixedeffects,
                              any.prior=FALSE,
+                             itemclass=1L,
                              prior.type=rep('none', length(startvalues[[i]])),
                              fixed.design=fixed.design.list[[i]],
-                             lbound=c(rep(-Inf, length(startvalues[[i]]) - 2),0,.5),
-                             ubound=c(rep(Inf, length(startvalues[[i]]) - 2),.5,1),
+                             lbound=rep(-Inf, length(startvalues[[i]])),
+                             ubound=rep(Inf, length(startvalues[[i]])),
                              prior_1=rep(NaN,length(startvalues[[i]])),
                              prior_2=rep(NaN,length(startvalues[[i]])))
             tmp2 <- parnumber:(parnumber + length(freepars[[i]]) - 1L)
@@ -193,10 +187,10 @@ LoadPars <- function(itemtype, itemloc, lambdas, zetas, guess, upper, fulldata,
                              ncat=K[i],
                              nfixedeffects=nfixedeffects,
                              any.prior=FALSE,
+                             itemclass=3L,
                              prior.type=rep('none', length(startvalues[[i]])),
                              fixed.design=fixed.design.list[[i]],
                              est=freepars[[i]],
-                             dat=fulldata[ ,tmp],
                              lbound=rep(-Inf, length(startvalues[[i]])),
                              ubound=rep(Inf, length(startvalues[[i]])),
                              prior_1=rep(NaN,length(startvalues[[i]])),
@@ -207,41 +201,20 @@ LoadPars <- function(itemtype, itemloc, lambdas, zetas, guess, upper, fulldata,
             parnumber <- parnumber + length(freepars[[i]])
             next
         }
-
-        if(itemtype[i] == '1PL' && K[i] > 2L){
-            pars[[i]] <- new('graded',
-                             par=startvalues[[i]],
-                             nfact=nfact,
-                             ncat=K[i],
-                             nfixedeffects=nfixedeffects,
-                             any.prior=FALSE,
-                             prior.type=rep('none', length(startvalues[[i]])),
-                             fixed.design=fixed.design.list[[i]],
-                             est=freepars[[i]],
-                             dat=fulldata[ ,tmp],
-                             lbound=rep(-Inf, length(startvalues[[i]])),
-                             ubound=rep(Inf, length(startvalues[[i]])),
-                             prior_1=rep(NaN,length(startvalues[[i]])),
-                             prior_2=rep(NaN,length(startvalues[[i]])))
-            tmp2 <- parnumber:(parnumber + length(freepars[[i]]) - 1L)
-            pars[[i]]@parnum <- tmp2
-            parnumber <- parnumber + length(freepars[[i]])
-            next
-        }
-
+        
         if(any(itemtype[i] == c('2PL', '3PL', '3PLu', '4PL'))){
             pars[[i]] <- new('dich',
                              par=startvalues[[i]],
                              est=freepars[[i]],
                              nfact=nfact,
+                             itemclass=1L,
                              nfixedeffects=nfixedeffects,
-                             dat=fulldata[ ,tmp],
                              ncat=2L,
                              any.prior=FALSE,
                              prior.type=rep('none', length(startvalues[[i]])),
                              fixed.design=fixed.design.list[[i]],
-                             lbound=c(rep(-Inf, length(startvalues[[i]]) - 2),0,.5),
-                             ubound=c(rep(Inf, length(startvalues[[i]]) - 2),.5,1),
+                             lbound=rep(-Inf, length(startvalues[[i]])),
+                             ubound=rep(Inf, length(startvalues[[i]])),
                              prior_1=rep(NaN,length(startvalues[[i]])),
                              prior_2=rep(NaN,length(startvalues[[i]])))
             tmp2 <- parnumber:(parnumber + length(freepars[[i]]) - 1L)
@@ -255,15 +228,15 @@ LoadPars <- function(itemtype, itemloc, lambdas, zetas, guess, upper, fulldata,
                              par=startvalues[[i]],
                              est=freepars[[i]],
                              nfact=nfact,
+                             itemclass=8L,
                              nfixedeffects=nfixedeffects,
-                             dat=fulldata[ ,tmp],
                              ncat=K[i],
                              correctcat=key[i],
                              any.prior=FALSE,
                              prior.type=rep('none', length(startvalues[[i]])),
                              fixed.design=fixed.design.list[[i]],
-                             lbound=c(rep(-Inf, nfact+1),0,.5, rep(-Inf, length(startvalues[[i]])-nfact-3)),
-                             ubound=c(rep(Inf, nfact+1),.5,1, rep(Inf, length(startvalues[[i]])-nfact-3)),
+                             lbound=rep(-Inf, length(startvalues[[i]])),
+                             ubound=rep(Inf, length(startvalues[[i]])),
                              prior_1=rep(NaN,length(startvalues[[i]])),
                              prior_2=rep(NaN,length(startvalues[[i]])))
             tmp2 <- parnumber:(parnumber + length(freepars[[i]]) - 1L)
@@ -279,10 +252,10 @@ LoadPars <- function(itemtype, itemloc, lambdas, zetas, guess, upper, fulldata,
                              ncat=K[i],
                              nfixedeffects=nfixedeffects,
                              any.prior=FALSE,
+                             itemclass=5L,
                              prior.type=rep('none', length(startvalues[[i]])),
                              fixed.design=fixed.design.list[[i]],
                              est=freepars[[i]],
-                             dat=fulldata[ ,tmp],
                              lbound=rep(-Inf, length(startvalues[[i]])),
                              ubound=rep(Inf, length(startvalues[[i]])),
                              prior_1=rep(NaN,length(startvalues[[i]])),
@@ -298,12 +271,12 @@ LoadPars <- function(itemtype, itemloc, lambdas, zetas, guess, upper, fulldata,
                              par=startvalues[[i]],
                              nfact=nfact,
                              ncat=K[i],
+                             itemclass=2L,
                              nfixedeffects=nfixedeffects,
                              any.prior=FALSE,
                              prior.type=rep('none', length(startvalues[[i]])),
                              fixed.design=fixed.design.list[[i]],
                              est=freepars[[i]],
-                             dat=fulldata[ ,tmp],
                              lbound=rep(-Inf, length(startvalues[[i]])),
                              ubound=rep(Inf, length(startvalues[[i]])),
                              prior_1=rep(NaN,length(startvalues[[i]])),
@@ -321,10 +294,10 @@ LoadPars <- function(itemtype, itemloc, lambdas, zetas, guess, upper, fulldata,
                              ncat=K[i],
                              nfixedeffects=nfixedeffects,
                              any.prior=FALSE,
+                             itemclass=3L,
                              prior.type=rep('none', length(startvalues[[i]])),
                              fixed.design=fixed.design.list[[i]],
                              est=freepars[[i]],
-                             dat=fulldata[ ,tmp],
                              lbound=rep(-Inf, length(startvalues[[i]])),
                              ubound=rep(Inf, length(startvalues[[i]])),
                              prior_1=rep(NaN,length(startvalues[[i]])),
@@ -341,12 +314,12 @@ LoadPars <- function(itemtype, itemloc, lambdas, zetas, guess, upper, fulldata,
                              par=startvalues[[i]],
                              nfact=nfact,
                              ncat=K[i],
+                             itemclass=6L,
                              nfixedeffects=nfixedeffects,
                              any.prior=FALSE,
                              prior.type=rep('none', length(startvalues[[i]])),
                              fixed.design=fixed.design.list[[i]],
                              est=freepars[[i]],
-                             dat=fulldata[ ,tmp],
                              lbound=rep(-Inf, length(startvalues[[i]])),
                              ubound=rep(Inf, length(startvalues[[i]])),
                              prior_1=rep(NaN,length(startvalues[[i]])),
@@ -364,11 +337,11 @@ LoadPars <- function(itemtype, itemloc, lambdas, zetas, guess, upper, fulldata,
                              est=freepars[[i]],
                              nfact=nfact,
                              ncat=K[i],
+                             itemclass=4L,
                              nfixedeffects=nfixedeffects,
                              any.prior=FALSE,
                              prior.type=rep('none', length(startvalues[[i]])),
                              fixed.design=fixed.design.list[[i]],
-                             dat=fulldata[ ,tmp],
                              lbound=rep(-Inf, length(startvalues[[i]])),
                              ubound=rep(Inf, length(startvalues[[i]])),
                              prior_1=rep(NaN,length(startvalues[[i]])),
@@ -385,13 +358,13 @@ LoadPars <- function(itemtype, itemloc, lambdas, zetas, guess, upper, fulldata,
                              est=freepars[[i]],
                              nfact=nfact,
                              ncat=2L,
+                             itemclass=7L,
                              nfixedeffects=nfixedeffects,
                              any.prior=FALSE,
                              prior.type=rep('none', length(startvalues[[i]])),
                              fixed.design=fixed.design.list[[i]],
-                             dat=fulldata[ ,tmp],
-                             lbound=c(rep(-Inf, length(startvalues[[i]]) - 2),0,.5),
-                             ubound=c(rep(Inf, length(startvalues[[i]]) - 2),.5,1),
+                             lbound=rep(-Inf, length(startvalues[[i]])),
+                             ubound=rep(Inf, length(startvalues[[i]])),
                              prior_1=rep(NaN,length(startvalues[[i]])),
                              prior_2=rep(NaN,length(startvalues[[i]])))
             tmp2 <- parnumber:(parnumber + length(freepars[[i]]) - 1L)
@@ -404,9 +377,9 @@ LoadPars <- function(itemtype, itemloc, lambdas, zetas, guess, upper, fulldata,
             pars[[i]] <- customItems[[itemtype[i] == names(customItems)]]
             pars[[i]]@nfact <- nfact
             pars[[i]]@ncat <- K[i]
-            pars[[i]]@nfixedeffects <- nfixedeffects            
-            pars[[i]]@dat <- fulldata[ ,tmp]
+            pars[[i]]@nfixedeffects <- nfixedeffects
             pars[[i]]@any.prior <- FALSE
+            pars[[i]]@itemclass <- 9L
             pars[[i]]@prior.type <- rep('none', length(pars[[i]]@par))
             pars[[i]]@prior_1 <- rep(NaN,length(pars[[i]]@par))
             pars[[i]]@prior_2 <- rep(NaN,length(pars[[i]]@par))
