@@ -1,7 +1,10 @@
 #' Compute profiled-likelihood confidence intervals
 #' 
 #' Computes profiled-likelihood based confidence intervals. Supports the inclusion of prior parameter 
-#' distributions as well as equality constraints. Only supports unidimensional models. 
+#' distributions as well as equality constraints. For multidimensional models, the CI's for the slopes 
+#' are not estimated due to the possibility of signs flipping during estimation. In unidimensional models, the 
+#' slope parameters are assumed to be greater than zero, and a lower bound is imposed to ensure that sign
+#' flipping does not occur.
 #' 
 #' @aliases PLCI.mirt
 #' @param mod a converged mirt model
@@ -45,29 +48,23 @@ PLCI.mirt <- function(mod, alpha = .05, parnum = NULL){
     f.min <- function(value, dat, model, which, sv, get.LL, large, parprior, parnames){        
         sv$est[which] <- FALSE
         sv$value[which] <- value
-        if(any(parnames[which] %in% paste0('d', 1L:10L))){            
-            intnum <- as.numeric(strsplit(as.character(parnames[which]), 'd')[[1L]][2L])
-            #test if graded model            
-            if(sv$name[which-intnum] != 'd0'){    
-                about <- parnums[which]
-                if(intnum != 1L) about <- sort(which:(which-intnum+1L))
-                len <- do.call(c, lapply(strsplit(as.character(parnames[(which+1L):(which+10L)]), 'd'), length))
-                if(which(len == 1L)[1L] > 1L)
-                    about <- c(about, max(about) + 1L:(which(len == 1L)[1L]-1L))
-                vals <- rep(value, length(about))
-                if(intnum == 1L){                    
-                    vals[2L:length(vals)] <- vals[2L:length(vals)] - seq(0.1, 0.5, length.out = length(vals)-1L)
-                } else if(intnum == length(about)){
-                    vals[1L:(length(vals)-1L)] <- vals[1L:(length(vals)-1L)] + 
-                        seq(0.5, 0.1, length.out = length(1L:(length(vals)-1L)))
+        if(sv$class[which] == 'graded'){
+            if(!(sv$name[which] %in% paste0('a', 1L:30L))){
+                itemname <- sv$item[which]
+                itemnum <- sv$parnum[which]
+                itemsv <- sv[sv$item == itemname & !(sv$name %in% paste0('a', 1L:30L)), ]
+                if(min(itemsv$parnum) == itemnum){
+                    ds <- c(value, seq(from = value-.1, to=value-1, length.out=nrow(itemsv)-1L))
+                } else if(max(itemsv$parnum) == itemnum){
+                    ds <- c(seq(from = value+1, to=value+.1, length.out=nrow(itemsv)-1L), value)
                 } else {
-                    vals[1L:(intnum-1L)] <- vals[1L:(intnum-1L)] + seq(0.5, 0.1, length.out = intnum-1L)
-                    vals[(intnum+1L):length(vals)] <- vals[(intnum+1L):length(vals)] -
-                        seq(0.1, 0.5, length.out = length((intnum+1L):length(vals)))                                        
+                    ds <- c(seq(from = value+1, to=value+.1, length.out=itemnum-itemsv$parnum[1L]),
+                            value, seq(from = value-.1, to=value-1, 
+                                       length.out=itemsv$parnum[nrow(itemsv)]-itemnum))
                 }
-                sv$value[about] <- vals
+                sv$value[itemsv$parnum] <- ds
             }
-        }            
+        }
         got.LL <- try(compute.LL(dat=dat, model=model, sv=sv, large=large, parprior=parprior), silent=TRUE)
         if(is(got.LL, 'try-error')) return(1e8)
         ret <- (got.LL - get.LL)^2
@@ -76,19 +73,22 @@ PLCI.mirt <- function(mod, alpha = .05, parnum = NULL){
     }
     
     LLpar <- function(parnum, parnums, parnames, lbound, ubound, dat, model, large, sv, get.LL, parprior){
-        lower <- ifelse(lbound[parnum] == -Inf, -5, lbound[parnum])
+        lower <- ifelse(lbound[parnum] == -Inf, -10, lbound[parnum])
         upper <- ifelse(ubound[parnum] == Inf, 10, ubound[parnum])
-        opt.lower <- optimize(f.min, lower = lower, upper = pars[parnum], dat=dat, model=model, large=large,
-                              which=parnums[parnum], sv=sv, get.LL=get.LL, parprior=parprior, parnames=parnames,
-                              tol = .01)        
-        opt.upper <- optimize(f.min, lower = pars[parnum], upper = upper, dat=dat, model=model, large=large,
-                              which=parnums[parnum], sv=sv, get.LL=get.LL, parprior=parprior, parnames=parnames,
-                              tol = .01)
+        mid <- pars[parnum]
+        if(mid > lower){
+            opt.lower <- optimize(f.min, lower = lower, upper = mid, dat=dat, model=model, large=large,
+                                  which=parnums[parnum], sv=sv, get.LL=get.LL, parprior=parprior, 
+                                  parnames=parnames, tol = .01)
+        } else opt.lower <- list(minimum = lower)
+        if(mid < upper){
+            opt.upper <- optimize(f.min, lower = mid, upper = upper, dat=dat, model=model, large=large,
+                                  which=parnums[parnum], sv=sv, get.LL=get.LL, parprior=parprior, 
+                                  parnames=parnames, tol = .01)
+        } else opt.upper <- list(minimum = upper)
         c(lower=opt.lower$minimum, upper=opt.upper$minimum)
     }
     
-    if(mod@nfact != 1L)
-        stop('Likelihood confidence intervals only available for unidimensional models.')
     dat <- mod@data
     model <- mod@model[[1L]]    
     parprior <- mod@parprior 
@@ -102,6 +102,7 @@ PLCI.mirt <- function(mod, alpha = .05, parnum = NULL){
         pars <- sv$value[tmp]    
         LL.upper.crit <- LL.lower.crit <- pars
         parnums <- sv$parnum[tmp]
+        itemtypes <- sv$class[tmp]
         parnames <- sv$name[tmp]
         lbound <- sv$lbound[tmp]
         ubound <- sv$ubound[tmp]
@@ -109,9 +110,20 @@ PLCI.mirt <- function(mod, alpha = .05, parnum = NULL){
         pars <- sv$value    
         pars <- LL.upper.crit <- LL.lower.crit <- pars[sv$est]    
         parnums <- sv$parnum[sv$est]
+        itemtypes <- sv$class[sv$est]
         parnames <- sv$name[sv$est]
         lbound <- sv$lbound[sv$est]
         ubound <- sv$ubound[sv$est]        
+    }    
+    if(mod@nfact > 1L && is.null(parnum)){
+        #don't estimate slopes of multidimensional
+        tmp <- !(sv$name %in% paste0('a', 1:20)) & sv$est
+        pars <- sv$value[tmp]
+        parnames <- sv$name[tmp]
+        itemtypes <- sv$class[tmp]
+        parnums <- sv$parnum[tmp]
+        lbound <- sv$lbound[tmp]
+        ubound <- sv$ubound[tmp]
     }
     LL <- mod@logLik
     get.LL <- LL - qchisq(1-alpha, 1)/2
@@ -125,6 +137,7 @@ PLCI.mirt <- function(mod, alpha = .05, parnum = NULL){
                            ubound=ubound, dat=dat, model=model, large=large, sv=sv, get.LL=get.LL, parprior=parprior))           
     }
     colnames(result) <- c(paste0('lower_', alpha/2*100), paste0('upper_', (1-alpha/2)*100)) 
-    ret <- data.frame(Item=sv$item[parnums], parnam=sv$name[parnums], value=pars, result, row.names=NULL)
+    ret <- data.frame(Item=sv$item[parnums], class=itemtypes, parnam=sv$name[parnums], value=pars, 
+                      result, row.names=NULL)
     ret
 }
