@@ -49,7 +49,7 @@
 #' \code{\link{wald}}, \code{\link{itemplot}}, \code{\link{fscores}}, \code{\link{fitIndices}},
 #' \code{\link{extract.item}}, \code{\link{iteminfo}}, \code{\link{testinfo}}, \code{\link{probtrace}},
 #' \code{\link{boot.mirt}}, \code{\link{imputeMissing}}, \code{\link{itemfit}}, \code{\link{mod2values}},
-#' \code{\link{read.mirt}}, \code{\link{simdata}}, \code{\link{createItem}}
+#' \code{\link{simdata}}, \code{\link{createItem}}
 #' @references
 #' 
 #' Cai, L. (2010). A two-tier full-information item factor analysis model with applications.
@@ -69,7 +69,7 @@
 #'
 #' @keywords models
 #' @usage
-#' bfactor(data, model, model2 = mirt.model(paste0('G = 1-', ncol(data)), quiet = TRUE), 
+#' bfactor(data, model, model2 = mirt.model(paste0('G = 1-', ncol(data))), 
 #' SE = FALSE, SE.type = 'SEM', group = NULL, verbose = TRUE, ...)
 #'
 #'
@@ -93,19 +93,21 @@
 #' coef(mod2)
 #' anova(mod1, mod2)
 #' 
-#' ## don't estimate specific factor for item 32, and use estimated mod1 as starting values
-#' sv <- mod2values(mod1)
-#' sv$value[220] <- 0
-#' sv$est[220] <- FALSE
-#' mod3 <- bfactor(data, specific, pars = sv) #with excellent starting values 
-#' 
-#' #without using starting values (required if SE.type = 'SEM')
+#' ## don't estimate specific factor for item 32
 #' specific[32] <- NA
 #' mod3 <- bfactor(data, specific) 
 #' anova(mod1, mod3)
 #' 
-#'
+#' # same, but decalred manually (not run)
+#' #sv <- mod2values(mod1)
+#' #sv$value[220] <- 0 #parameter 220 is the 32 items specific slope
+#' #sv$est[220] <- FALSE
+#' #mod3 <- bfactor(data, specific, pars = sv) #with excellent starting values 
+#' 
+#' 
 #' #########
+#' # mixed itemtype example 
+#' 
 #' #simulate data
 #' a <- matrix(c(
 #' 1,0.5,NA,
@@ -184,13 +186,13 @@
 #'     G2 = 9-16
 #'     COV = G1*G2')
 #'     
-#' simmod <- bfactor(dataset, specific, model, quadpts = 15)
+#' simmod <- bfactor(dataset, specific, model, quadpts = 9, technical = list(TOL = 1e-3))
 #' coef(simmod)
 #' summary(simmod)
 #'
 #'     }
 #'
-bfactor <- function(data, model, model2 = mirt.model(paste0('G = 1-', ncol(data)), quiet = TRUE), 
+bfactor <- function(data, model, model2 = mirt.model(paste0('G = 1-', ncol(data))), 
                     SE = FALSE, SE.type = 'SEM', group = NULL, verbose = TRUE, ...)
 {
     Call <- match.call()
