@@ -53,8 +53,8 @@ setMethod(
 	signature = signature(object = 'ExploratoryClass'),
 	definition = function(object, draws = 5000, G2 = TRUE)
 	{
-        LLdraws <- function(LLDUMMY=NULL, nfact, N, grp, prodlist, fulldata, object, J, random, ot,
-                            PROBTRACE){
+        LLdraws <- function(LLDUMMY=NULL, nfact, N, grp, prodlist, fulldata, object, J, random, ot, 
+                            NO.CUSTOM){
             theta <- mvtnorm::rmvnorm(N,grp$gmeans, grp$gcov)            
             if(length(prodlist) > 0L)
                 theta <- prodterms(theta,prodlist)            
@@ -65,9 +65,8 @@ setMethod(
                 }
                 ot <- OffTerm(random, J=J, N=N)                
             }
-            itemtrace <- matrix(0, ncol=ncol(fulldata), nrow=N)            
-            for (i in 1L:J) itemtrace[ ,itemloc[i]:(itemloc[i+1L] - 1L)] <-
-                PROBTRACE[[i]](x=pars[[i]], Theta=theta, ot=ot[,i])
+            itemtrace <- computeItemtrace(pars=pars, Theta=theta, itemloc=itemloc, offterm=ot, 
+                                          NO.CUSTOM=NO.CUSTOM)
             return(exp(rowSums(log(itemtrace)*fulldata)))
         }           
         pars <- object@pars
@@ -83,15 +82,13 @@ setMethod(
         if(length(object@random) == 0L){
             ot <- matrix(0, 1, J)
         } else ot <- OffTerm(object@random, J=J, N=N)
-        PROBTRACE <- vector('list', J)
-        for(i in 1L:J)
-            PROBTRACE[[i]] <- selectMethod(ProbTrace, c(class(pars[[i]]), 'matrix'))        
+        NO.CUSTOM <- !any(sapply(pars, class) %in% 'custom')
         if(!is.null(globalenv()$MIRTCLUSTER)){
             LL <- parallel::parApply(cl=globalenv()$MIRTCLUSTER, LL, MARGIN=1, FUN=LLdraws, nfact=nfact, 
                                      N=N, grp=grp, prodlist=prodlist, fulldata=fulldata, object=object, J=J, 
-                                     random=object@random, ot=ot, PROBTRACE=PROBTRACE)
+                                     random=object@random, ot=ot, NO.CUSTOM=NO.CUSTOM)
         } else for(draw in 1L:draws)
-            LL[ ,draw] <- LLdraws(nfact=nfact, N=N, grp=grp, prodlist=prodlist, PROBTRACE=PROBTRACE,
+            LL[ ,draw] <- LLdraws(nfact=nfact, N=N, grp=grp, prodlist=prodlist, NO.CUSTOM=NO.CUSTOM,
                                   fulldata=fulldata, object=object, J=J, random=object@random, ot=ot)
         LL[is.nan(LL)] <- 0
         rwmeans <- rowMeans(LL)
@@ -130,19 +127,14 @@ setMethod(
                 nconstr <- nconstr + length(object@constrain[[i]]) - 1L
         nfact <- object@nfact - length(prodlist)
         nmissingtabdata <- sum(is.na(rowSums(object@tabdata)))
-        df <- length(r) - nestpars + nconstr + nfact*(nfact - 1)/2 - 1 - nmissingtabdata
-		AIC <- (-2) * logLik + 2 * (length(r) - df - 1)
-		BIC <- (-2) * logLik + (length(r) - df - 1)*log(N)
 		if(G2){
 			if(any(is.na(data))){
-			    object@G2 <- object@X2 <- NaN
+			    object@G2 <- NaN
 			} else {
                 r <- r[!is.na(expected)]
                 expected <- expected[!is.na(expected)]
 				G2 <- 2 * sum(r*log(r/(sum(r)*expected)))
-                X2 <- sum((r - sum(r)*expected)^2 / (sum(r)*expected))
 				object@G2 <- G2
-                object@X2 <- X2
                 if(logLikpre == 0){
     				null.mod <- object@null.mod
     				object@TLI <- (null.mod@G2 / null.mod@df - G2/df) / (null.mod@G2 / null.mod@df - 1)
@@ -153,9 +145,6 @@ setMethod(
         if(logLikpre < 0)
             object@logLik <- logLikpre
 		object@SElogLik <- SElogLik
-		object@AIC <- AIC
-		object@BIC <- BIC
-		object@df <- as.integer(df)
 		return(object)
 	}
 )
