@@ -6,8 +6,8 @@ setMethod(
 	                      returnER = FALSE, verbose = TRUE, gmean, gcov, scores.only)
 	{
 	    #local functions for apply
-	    MAP <- function(ID, scores, pars, tabdata, itemloc, gp, prodlist, PROBTRACE){
-	        estimate <- try(nlm(MAP.mirt,scores[ID, ],pars=pars, patdata=tabdata[ID, ], PROBTRACE=PROBTRACE,
+	    MAP <- function(ID, scores, pars, tabdata, itemloc, gp, prodlist){
+	        estimate <- try(nlm(MAP.mirt,scores[ID, ],pars=pars, patdata=tabdata[ID, ], 
 	                            itemloc=itemloc, gp=gp, prodlist=prodlist, hessian=TRUE))
 	        if(is(estimate, 'try-error')) 
 	            return(rep(NA, ncol(scores)*2))	        
@@ -15,10 +15,10 @@ setMethod(
 	        if(is(SEest, 'try-error')) SEest <- rep(NA, ncol(scores))
 	        return(c(estimate$estimate, SEest))
 	    }
-	    ML <- function(ID, scores, pars, tabdata, itemloc, gp, prodlist, PROBTRACE){
+	    ML <- function(ID, scores, pars, tabdata, itemloc, gp, prodlist){
 	        if(any(scores[ID, ] %in% c(-Inf, Inf)))
                 return(c(scores[ID, ], rep(NA, ncol(scores))))	        
-	        estimate <- try(nlm(MAP.mirt,scores[ID, ],pars=pars,patdata=tabdata[ID, ], PROBTRACE=PROBTRACE,
+	        estimate <- try(nlm(MAP.mirt,scores[ID, ],pars=pars,patdata=tabdata[ID, ], 
 	                            itemloc=itemloc, gp=gp, prodlist=prodlist, ML=TRUE, hessian=TRUE))
 	        if(is(estimate, 'try-error')) 
 	            return(rep(NA, ncol(scores)*2))
@@ -26,13 +26,16 @@ setMethod(
 	        if(is(SEest, 'try-error')) SEest <- rep(NA, ncol(scores))
 	        return(c(estimate$estimate, SEest))
 	    }
-	    WLE <- function(ID, scores, pars, tabdata, itemloc, gp, prodlist, degrees, PROBTRACE){            
-	        estimate <- try(nlm(gradnorm.WLE,scores[ID, ],pars=pars,patdata=tabdata[ID, ], PROBTRACE=PROBTRACE,
-	                            itemloc=itemloc, gp=gp, prodlist=prodlist, degrees=degrees,
-	                            hessian = TRUE))
+	    WLE <- function(ID, scores, pars, tabdata, itemloc, gp, prodlist, degrees){            
+	        estimate <- try(nlm(gradnorm.WLE,scores[ID, ],pars=pars,patdata=tabdata[ID, ],
+	                            itemloc=itemloc, gp=gp, prodlist=prodlist, degrees=degrees))
 	        if(is(estimate, 'try-error')) 
 	            return(rep(NA, ncol(scores)*2))
-	        return(c(estimate$estimate, rep(NA, ncol(scores))))
+            TI <- 0
+            for(i in 1L:(length(itemloc)-1L)) 
+                TI <- TI + iteminfo(pars[[i]], Theta=estimate$estimate, degrees=degrees)
+	        SEest <- 1 / sqrt(TI)
+	        return(c(estimate$estimate, SEest))
 	    }
 	    EAP <- function(ID, log_itemtrace, tabdata, ThetaShort, W){
 	        L <- rowSums(log_itemtrace[ ,as.logical(tabdata[ID,]), drop = FALSE])
@@ -53,6 +56,7 @@ setMethod(
             mins <- apply(object@data, 2, min, na.rm=TRUE)
             response.pattern <- response.pattern - matrix(mins, nrow(response.pattern), 
                                                           ncol(response.pattern), byrow=TRUE)
+            colnames(response.pattern) <- colnames(object@data)
             newmod <- mirt(response.pattern, nfact, pars=sv, calcNull=FALSE, 
                            technical=list(customK=object@K, override.df=TRUE))
             ret <- fscores(newmod, rotate=rotate, full.scores=full.scores, scores.only=scores.only,
@@ -104,10 +108,7 @@ setMethod(
 		SEscores <- scores <- matrix(0, nrow(tabdata), nfact)
 		W <- mvtnorm::dmvnorm(ThetaShort,gp$gmeans,gp$gcov)
 		W <- W/sum(W)
-	    PROBTRACE <- vector('list', J)
-	    for(i in 1L:J)
-	        PROBTRACE[[i]] <- selectMethod(ProbTrace, c(class(pars[[i]]), 'matrix'))
-        itemtrace <- computeItemtrace(pars=pars, Theta=Theta, itemloc=itemloc, PROBTRACE=PROBTRACE)
+        itemtrace <- computeItemtrace(pars=pars, Theta=Theta, itemloc=itemloc)
 	    log_itemtrace <- log(itemtrace)
 	    if(!is.null(globalenv()$MIRTCLUSTER)){
 	        tmp <- t(parallel::parApply(cl=globalenv()$MIRTCLUSTER, matrix(1:nrow(scores)), 1, EAP, 
@@ -123,10 +124,10 @@ setMethod(
             if(!is.null(globalenv()$MIRTCLUSTER)){
                 tmp <- t(parallel::parApply(cl=globalenv()$MIRTCLUSTER, matrix(1:nrow(scores)), 1, MAP, 
                                      scores=scores, pars=pars, tabdata=tabdata, itemloc=itemloc, 
-                                     gp=gp, prodlist=prodlist, PROBTRACE=PROBTRACE))
+                                     gp=gp, prodlist=prodlist))
             } else {
                 tmp <- t(apply(matrix(1:nrow(scores)), 1, MAP, scores=scores, pars=pars, 
-                             tabdata=tabdata, itemloc=itemloc, gp=gp, prodlist=prodlist, PROBTRACE=PROBTRACE))
+                             tabdata=tabdata, itemloc=itemloc, gp=gp, prodlist=prodlist))
             }            
             scores <- tmp[ ,1:nfact, drop = FALSE]
             SEscores <- tmp[ ,-c(1:nfact), drop = FALSE]			
@@ -145,24 +146,24 @@ setMethod(
             if(!is.null(globalenv()$MIRTCLUSTER)){
                 tmp <- t(parallel::parApply(cl=globalenv()$MIRTCLUSTER, matrix(1:nrow(scores)), 1, ML, 
                                             scores=scores, pars=pars, tabdata=tabdata, itemloc=itemloc, 
-                                            gp=gp, prodlist=prodlist, PROBTRACE=PROBTRACE))
+                                            gp=gp, prodlist=prodlist))
             } else {
                 tmp <- t(apply(matrix(1:nrow(scores)), 1, ML, scores=scores, pars=pars, 
-                               tabdata=tabdata, itemloc=itemloc, gp=gp, prodlist=prodlist, PROBTRACE=PROBTRACE))
+                               tabdata=tabdata, itemloc=itemloc, gp=gp, prodlist=prodlist))
             }            
             scores <- tmp[ ,1:nfact, drop = FALSE]
             SEscores <- tmp[ ,-c(1:nfact), drop = FALSE]			
 		}
         if(method == 'WLE'){              
-            itemtrace <- computeItemtrace(pars=pars, Theta=Theta, itemloc=itemloc, PROBTRACE=PROBTRACE)
+            itemtrace <- computeItemtrace(pars=pars, Theta=Theta, itemloc=itemloc)
             if(!is.null(globalenv()$MIRTCLUSTER)){
                 tmp <- t(parallel::parApply(cl=globalenv()$MIRTCLUSTER, matrix(1:nrow(scores)), 1, WLE, 
                                             scores=scores, pars=pars, tabdata=tabdata, itemloc=itemloc, 
-                                            gp=gp, prodlist=prodlist, degrees=degrees, PROBTRACE=PROBTRACE))
+                                            gp=gp, prodlist=prodlist, degrees=degrees))
             } else {
                 tmp <- t(apply(matrix(1:nrow(scores)), 1, WLE, scores=scores, pars=pars, 
                                tabdata=tabdata, itemloc=itemloc, gp=gp, prodlist=prodlist, 
-                               degrees=degrees, PROBTRACE=PROBTRACE))
+                               degrees=degrees))
             }            
             scores <- tmp[ ,1:nfact, drop = FALSE]   
             SEscores <- tmp[ ,-c(1:nfact), drop = FALSE]
@@ -262,23 +263,20 @@ setMethod(
 )
 
 # MAP scoring for mirt
-MAP.mirt <- function(Theta, pars, patdata, itemloc, gp, prodlist, ML=FALSE, PROBTRACE)
+MAP.mirt <- function(Theta, pars, patdata, itemloc, gp, prodlist, ML=FALSE)
 {
     ThetaShort <- Theta
     Theta <- matrix(Theta, nrow=1)
     if(length(prodlist) > 0L)
         Theta <- prodterms(Theta,prodlist)
-    itemtrace <- matrix(0, ncol=length(patdata), nrow=nrow(Theta))
-    for (i in 1L:(length(itemloc)-1L))
-        itemtrace[ ,itemloc[i]:(itemloc[i+1L] - 1L)] <- PROBTRACE[[i]](x=pars[[i]], Theta=Theta)
-    itemtrace[itemtrace < 1e-8] <- 1e-8
+    itemtrace <- computeItemtrace(pars=pars, Theta=Theta, itemloc=itemloc)
     L <- sum(log(itemtrace)[as.logical(patdata)])
     prior <- mvtnorm::dmvnorm(ThetaShort, gp$gmeans, gp$gcov)
     L <- ifelse(ML, -L, (-1)*(L + log(prior)))
     L
 }
 
-gradnorm.WLE <- function(Theta, pars, patdata, itemloc, gp, prodlist, degrees, PROBTRACE){
+gradnorm.WLE <- function(Theta, pars, patdata, itemloc, gp, prodlist, degrees){
     ThetaShort <- Theta
     Theta <- matrix(Theta, nrow=1)
     if(length(prodlist) > 0L)
@@ -290,8 +288,8 @@ gradnorm.WLE <- function(Theta, pars, patdata, itemloc, gp, prodlist, degrees, P
         dP[[i]] <- d2P[[i]] <- itemtrace
     I <- numeric(1)
     dW <- dL <- numeric(nfact)
-    for (i in 1L:(length(itemloc)-1L)){
-        itemtrace[ ,itemloc[i]:(itemloc[i+1L] - 1L)] <- PROBTRACE[[i]](x=pars[[i]], Theta=Theta)
+    itemtrace <- computeItemtrace(pars=pars, Theta=Theta, itemloc=itemloc)
+    for (i in 1L:(length(itemloc)-1L)){        
         deriv <- DerivTheta(x=pars[[i]], Theta=Theta)
         for(k in 1L:nfact){
             dPitem <- d2Pitem <- matrix(0, 1, length(deriv[[1L]]))
@@ -346,9 +344,7 @@ EAPsum <- function(x, full.scores = FALSE, quadpts = NULL, S_X2 = FALSE, gp){
     K <- x@K
     J <- length(K)
     itemloc <- x@itemloc
-    itemtrace <- matrix(0, ncol=ncol(x@tabdatalong)-1, nrow=nrow(Theta))
-    for (i in 1L:J)
-        itemtrace[ ,itemloc[i]:(itemloc[i+1L] - 1L)] <- ProbTrace(x=pars[[i]], Theta=Theta)
+    itemtrace <- computeItemtrace(pars=pars, Theta=Theta, itemloc=itemloc)
     itemtrace <- t(itemtrace)
     tmp <- calcL1(itemtrace=itemtrace, K=K, itemloc=itemloc)
     L1 <- tmp$L1
