@@ -17,6 +17,7 @@
 #' @param CE logical; plot confidence envelope?
 #' @param CEalpha area remaining in the tail for confidence envelope. Default gives 95\% confidence region
 #' @param CEdraws draws number of draws to use for confidence envelope
+#' @param rot a list of rotation coordinates to be used for 3 dimensional plots
 #' @param drop.zeros logical; drop slope values that are numerically close to zero to reduce dimensionality?
 #' Useful in objects returned from \code{\link{bfactor}} or other confirmatory models that contain several 
 #' zero slopes
@@ -30,7 +31,7 @@
 #' data(LSAT7)
 #' fulldata <- expand.table(LSAT7)
 #' mod1 <- mirt(fulldata,1,SE=TRUE)
-#' mod2 <- mirt(fulldata,1, itemtype = '1PL')
+#' mod2 <- mirt(fulldata,1, itemtype = 'Rasch')
 #' mod3 <- mirt(fulldata,2)
 #'
 #' itemplot(mod1, 2)
@@ -52,13 +53,15 @@
 #'     }
 #'
 itemplot <- function(object, item, type = 'trace', degrees = 45, CE = FALSE, CEalpha = .05,
-                     CEdraws = 1000, drop.zeros = FALSE, ...){
+                     CEdraws = 1000, drop.zeros = FALSE, rot = list(xaxis = -70, yaxis = 30, zaxis = 10), 
+                     ...){
     if(is.list(object)) inames <- colnames(object[[1]]@data)
     else inames <- colnames(object@data)
     ind <- 1:length(inames)
     if(!is.numeric(item)) item <- ind[inames == item]
+    rot <- list(x = rot[[1]], y = rot[[2]], z = rot[[3]])
     ret <- itemplot.internal(object=object, item=item, type=type, degrees=degrees, CE=CE,
-                             CEalpha=CEalpha, CEdraws=CEdraws, drop.zeros=drop.zeros, ...)
+                             CEalpha=CEalpha, CEdraws=CEdraws, drop.zeros=drop.zeros, rot=rot, ...)
     if(is.null(ret)) return(invisible(ret))
     else return(ret)
 }
