@@ -18,18 +18,16 @@
 #' fitted,MultipleGroupClass-method
 #' @param data a \code{matrix} or \code{data.frame} that consists of
 #' numerically ordered data, with missing data coded as \code{NA}
-#' @param model an object or named list of objects returned from \code{mirt.model()} declaring how
-#' the factor model is to be estimated. The names of the list input must correspond to the unique values
-#' in the \code{group} variable. See \code{\link{mirt.model}} for more details
+#' @param model a single model object returned from \code{mirt.model()} declaring how
+#' the factor model is to be estimated. See \code{\link{mirt.model}} for more details
 #' @param group a character vector indicating group membership
 #' @param invariance a character vector containing the following possible options:
 #' \describe{
 #' \item{\code{'free_means'}}{for freely estimating all latent means (reference group constrained to 0)}
-#' \item{\code{'free_varcov'}}{for freely estimating the variance-covariance matrix across groups
-#' (reference group has variances equal to 1, but
-#' freely estimated covariance terms if specified in the model)}
-#' \item{\code{'covariances'}}{to constrain all the covariance parameters to be equal, note that this only
-#' makes sense if the factor variances are the same (i.e., unity)}
+#' \item{\code{'free_var'}}{for freely estimating all latent variances (reference group constrained to 1's)}
+#' \item{\code{'free_cov'}}{for freely estimating all latent covariances (reference group constrained to an 
+#' Identity matrix)}
+#' \item{\code{'free_varcov'}}{calls both \code{'free_var'} and \code{'free_cov'}}
 #' \item{\code{'slopes'}}{to constrain all the slopes to be equal across all groups}
 #' \item{\code{'intercepts'}}{to constrain all the intercepts to be equal across all groups, note for
 #' nominal models this also includes the category specific slope parameters}}
@@ -65,6 +63,7 @@
 #' @param y an unused variable to be ignored
 #' @param key see \code{\link{mirt}} for details
 #' @param itemtype see \code{\link{mirt}} for details
+#' @param CI see \code{\link{mirt}} for details
 #' @param constrain see \code{\link{mirt}} for details
 #' @param grsm.block see \code{\link{mirt}} for details
 #' @param rsm.block see \code{\link{mirt}} for details
@@ -74,21 +73,8 @@
 #' @param object2 an object of class \code{confmirtClass}
 #' @param digits the number of significant digits to be rounded
 #' @param ... additional arguments to be passed
-#' @param technical list specifying subtle parameters that can be adjusted. These
-#' values are
-#' \describe{
-#' \item{NCYCLES}{max number of cycles; default 2000 for MHRM and 500 for EM}
-#' \item{MAXQUAD}{maximum number of quadratures; default 10000}
-#' \item{BURNIN}{number of burn in cycles (stage 1); default 150}
-#' \item{SEMCYCLES}{number of SEM cycles (stage 2); default 50}
-#' \item{TOL}{minimum threshold tolerance for convergence. If MH-RM, must occur on three consecutive
-#' occations; default is .001 for MH-RM and .0001 for EM}
-#'   \item{set.seed}{seed number used during estimation. Default is 12345}
-#'   \item{gain}{a vector of three values specifying the numerator, exponent, and subtracted
-#'      values for the RM gain value. Default is \code{c(0.05,0.5,0.004)}}
-#'  \item{return_newconstrain}{if \code{TRUE} returns a list consisting of the constraints to be used
-#'  just before estimation begins}
-#' }
+#' @param technical list specifying subtle parameters that can be adjusted. See 
+#' \code{\link{mirt}} for details
 #' @author Phil Chalmers \email{rphilip.chalmers@@gmail.com}
 #' @seealso
 #' \code{\link{expand.table}}, \code{\link{key2binary}}, \code{\link{mirt.model}}, \code{\link{mirt}},
@@ -96,7 +82,7 @@
 #' \code{\link{wald}}, \code{\link{itemplot}}, \code{\link{fscores}}, \code{\link{fitIndices}},
 #' \code{\link{extract.item}}, \code{\link{iteminfo}}, \code{\link{testinfo}}, \code{\link{probtrace}},
 #' \code{\link{boot.mirt}}, \code{\link{imputeMissing}}, \code{\link{itemfit}}, \code{\link{mod2values}},
-#' \code{\link{read.mirt}}, \code{\link{simdata}}, \code{\link{createItem}}, \code{\link{mirtCluster}}
+#' \code{\link{simdata}}, \code{\link{createItem}}, \code{\link{mirtCluster}}
 #' @keywords models
 #' @usage
 #' multipleGroup(data, model, group, itemtype = NULL, guess = 0, upper = 1, SE = FALSE, SE.type = 'SEM',
@@ -104,7 +90,7 @@
 #' parprior = NULL, calcNull = TRUE, draws = 5000, quadpts = NULL, grsm.block = NULL, rsm.block = NULL,
 #' key = NULL, technical = list(), accelerate = TRUE, verbose = TRUE, ...)
 #'
-#' \S4method{coef}{MultipleGroupClass}(object, digits = 3, verbose = TRUE, ...)
+#' \S4method{coef}{MultipleGroupClass}(object, CI = .95, digits = 3, verbose = TRUE, ...)
 #'
 #' \S4method{summary}{MultipleGroupClass}(object, digits = 3, verbose = TRUE, ...)
 #'
@@ -137,9 +123,9 @@
 #' mod_metric <- multipleGroup(dat, models, group = group, invariance=c('slopes')) #equal slopes
 #' #equal intercepts, free variance and means
 #' mod_scalar2 <- multipleGroup(dat, models, group = group, 
-#'                              invariance=c('slopes', 'intercepts', 'free_varcov','free_means'))
+#'                              invariance=c('slopes', 'intercepts', 'free_var','free_means'))
 #' mod_scalar1 <- multipleGroup(dat, models, group = group,  #fixed means
-#'                              invariance=c('slopes', 'intercepts', 'free_varcov'))
+#'                              invariance=c('slopes', 'intercepts', 'free_var'))
 #' mod_fullconstrain <- multipleGroup(dat, models, group = group,
 #'                              invariance=c('slopes', 'intercepts'))
 #'
@@ -163,6 +149,13 @@
 #' constrain <- list(c(1, 63), c(5,67), c(9,71), c(13,75), c(17,79), c(21,83))
 #' equalslopes <- multipleGroup(dat, models, group = group, constrain = constrain)
 #' anova(equalslopes, mod_configural)
+#' 
+#' #same as above, but using mirt.model syntax
+#' newmodel <- mirt.model('
+#'     F = 1-15
+#'     CONSTRAINB = (1-6, a1)')
+#' equalslopes <- multipleGroup(dat, newmodel, group = group)
+#' coef(equalslopes)
 #'
 #' #############
 #' #DIF test for each item (using all other items as anchors)
@@ -173,7 +166,7 @@
 #' #loop over items (in practice, run in parallel to increase speed)
 #' estmodels <- vector('list', ncol(dat))
 #' for(i in 1:ncol(dat))
-#'     estmodels[[i]] <- multipleGroup(dat, models, group = group, verbose = FALSE,
+#'     estmodels[[i]] <- multipleGroup(dat, models, group = group, verbose = FALSE, calcNull=FALSE,
 #'                              invariance=c('free_means', 'free_varcov', itemnames[-i]))
 #'
 #' (anovas <- lapply(estmodels, anova, object2=refmodel))
@@ -186,7 +179,7 @@
 #' #constrain all intercepts
 #' estmodels <- vector('list', ncol(dat))
 #' for(i in 1:ncol(dat))
-#'     estmodels[[i]] <- multipleGroup(dat, models, group = group, verbose = FALSE, 
+#'     estmodels[[i]] <- multipleGroup(dat, models, group = group, verbose = FALSE, calcNull=FALSE,
 #'                              invariance=c('free_means', 'free_varcov', 'intercepts', 
 #'                              itemnames[-i]))
 #'
@@ -209,19 +202,17 @@
 #' group <- c(rep('D1', N), rep('D2', N))
 #'
 #' #group models
-#' model <- mirt.model()
+#' model <- mirt.model('
 #'    F1 = 1-5
 #'    F2 = 6-10
-#'    F3 = 11-15
-#'
-#' models <- list(D1=model, D2=model) #note the names match the groups
+#'    F3 = 11-15')
 #'
 #' #EM approach (not as accurate with 3 factors, but generally good for quick model comparisons)
-#' mod_configural <- multipleGroup(dat, models, group = group) #completely separate analyses
-#' mod_metric <- multipleGroup(dat, models, group = group, invariance=c('slopes')) #equal slopes
-#' mod_scalar <- multipleGroup(dat, models, group = group, #equal means, slopes, intercepts
+#' mod_configural <- multipleGroup(dat, model, group = group) #completely separate analyses
+#' mod_metric <- multipleGroup(dat, model, group = group, invariance=c('slopes')) #equal slopes
+#' mod_scalar <- multipleGroup(dat, model, group = group, #equal means, slopes, intercepts
 #'                              invariance=c('slopes', 'intercepts', 'free_varcov'))
-#' mod_fullconstrain <- multipleGroup(dat, models, group = group, #equal means, slopes, intercepts
+#' mod_fullconstrain <- multipleGroup(dat, model, group = group, #equal means, slopes, intercepts
 #'                              invariance=c('slopes', 'intercepts'))
 #'
 #' anova(mod_metric, mod_configural)
@@ -229,6 +220,9 @@
 #' anova(mod_fullconstrain, mod_scalar)
 #'
 #' #same as above, but with MHRM (more accurate with 3 factors, but slower)
+#' #define mirt cluster to compute log-likelihoods faster
+#' mirtCluster()
+#' 
 #' #completely separate analyses
 #' mod_configural <- multipleGroup(dat, models, group = group, method = 'MHRM') 
 #' #equal slopes
@@ -284,9 +278,18 @@ multipleGroup <- function(data, model, group, itemtype = NULL, guess = 0, upper
                           key = NULL, technical = list(), accelerate = TRUE, verbose = TRUE, ...)
 {
     Call <- match.call()
-    invariance.check <- invariance %in% c('free_means', 'free_varcov')
-    if(all(invariance.check) && length(constrain) == 0)
-        stop('Model is not identified without further constrains (may require additional anchoring items).')
+    if(length(model) > 1L) 
+        stop('multipleGroup only supports single group inputs')
+    invariance.check <- invariance %in% c('free_means', 'free_var', 'free_varcov')
+    if(all(invariance.check) && length(constrain) == 0){
+        warn <- TRUE
+        if(is(model, 'mirt.model')){
+            if(any(model$x[,1L] == 'CONSTRAINB'))
+                warn <- FALSE
+        }
+        if(warn)
+            stop('Model is not identified without further constrains (may require additional anchoring items).')
+    }
     mod <- ESTIMATION(data=data, model=model, group=group, invariance=invariance,
                       itemtype=itemtype, guess=guess, upper=upper,
                       pars=pars, constrain=constrain, SE=SE, grsm.block=grsm.block,
